@@ -2,7 +2,6 @@
    file Copyright.txt or https://cmake.org/licensing for details.  */
 #include "cmTargetLinkLibrariesCommand.h"
 
-#include <cstring>
 #include <memory>
 #include <sstream>
 #include <unordered_set>
@@ -11,9 +10,11 @@
 #include "cmExecutionStatus.h"
 #include "cmGeneratorExpression.h"
 #include "cmGlobalGenerator.h"
+#include "cmListFileCache.h"
 #include "cmMakefile.h"
 #include "cmMessageType.h"
 #include "cmPolicies.h"
+#include "cmProperty.h"
 #include "cmState.h"
 #include "cmStateTypes.h"
 #include "cmStringAlgorithms.h"
@@ -277,12 +278,12 @@ bool cmTargetLinkLibrariesCommand(std::vector<std::string> const& args,
       // with old versions of CMake and new)
       llt = GENERAL_LibraryType;
       std::string linkType = cmStrCat(args[0], "_LINK_TYPE");
-      const char* linkTypeString = mf.GetDefinition(linkType);
+      cmProp linkTypeString = mf.GetDefinition(linkType);
       if (linkTypeString) {
-        if (strcmp(linkTypeString, "debug") == 0) {
+        if (*linkTypeString == "debug") {
           llt = DEBUG_LibraryType;
         }
-        if (strcmp(linkTypeString, "optimized") == 0) {
+        if (*linkTypeString == "optimized") {
           llt = OPTIMIZED_LibraryType;
         }
       }
@@ -386,7 +387,7 @@ bool TLL::HandleLibrary(ProcessingState currentProcessingState,
     ? cmTarget::KeywordTLLSignature
     : cmTarget::PlainTLLSignature;
   if (!this->Target->PushTLLCommandTrace(
-        sig, this->Makefile.GetExecutionContext())) {
+        sig, this->Makefile.GetBacktrace().Top())) {
     std::ostringstream e;
     const char* modal = nullptr;
     MessageType messageType = MessageType::AUTHOR_WARNING;
@@ -564,7 +565,7 @@ void TLL::AffectsProperty(std::string const& prop)
   if (!this->EncodeRemoteReference) {
     return;
   }
-  // Add a wrapper to the expression to tell LookupLinkItems to look up
+  // Add a wrapper to the expression to tell LookupLinkItem to look up
   // names in the caller's directory.
   if (this->Props.insert(prop).second) {
     this->Target->AppendProperty(prop, this->DirectoryId);
